{
  Merge multiple .ccb files
}
{$I-,X+}

TYPE
  TNewSimHeader = RECORD
                    magic : ARRAY[0..7] OF CHAR; { CCbsim0^Z }
                    count : INTEGER;
                  END;

TYPE
  TSimObject = RECORD
                 name                 : ARRAY[1..20] OF CHAR;
                 damage, crew         : INTEGER;
                 id                   : INTEGER;
                 owner, picture       : INTEGER;
                 beamt, beamc         : INTEGER;
                 bays                 : INTEGER;
                 torpt, ammo, torpc   : INTEGER;
                 engine, hull, shield : INTEGER;
                 fc                   : ARRAY[1..3] OF CHAR;
                 agg, mass, flags     : INTEGER;
                 flakt, flakc         : LONGINT;
               END;

CONST
  MAX_SHIPS   = 200;
  MAX_ID      = 999;
  MAX_VERSION = 3;

VAR
  planet       : TSimObject;
  ships        : ARRAY[1..MAX_SHIPS] OF TSimObject;
  num_ships    : INTEGER;
  have_planet  : BOOLEAN;
  file_version : INTEGER;

CONST
  record_size: ARRAY[0..MAX_VERSION] OF WORD = (53,53,57,65);

{ Add a ship to record }
PROCEDURE AddShip(CONST o : TSimObject);
VAR i : INTEGER;
BEGIN
  { validate }
  IF (o.id <= 0) OR (o.id > MAX_ID) THEN BEGIN
    Writeln('WARNING: ship has invalid Id ', o.id, ', ignoring.');
    Exit;
  END;

  { check whether there is already a ship with the same Id }
  FOR i:=1 TO num_ships DO IF ships[i].id = o.id THEN BEGIN
    ships[i] := o;
    Exit;
  END;

  { new ship }
  IF num_ships >= MAX_SHIPS THEN BEGIN
    Writeln('WARNING: dropping ship ', o.id, '; too many ships in combined file.');
    Exit;
  END;

  Inc(num_ships);
  ships[num_ships] := o;
END;

{ Add a planet }
PROCEDURE AddPlanet(CONST o : TSimObject);
BEGIN
  IF (o.id <= 0) OR (o.id > MAX_ID) THEN BEGIN
    Writeln('WARNING: planet has invalid Id ', o.id, ', ignoring.');
    Exit;
  END;
  planet := o;
  have_planet := TRUE;
END;

{ Load a file }
PROCEDURE LoadFile(CONST name : STRING);
VAR
  F            : FILE;
  h            : TNewSimHeader;
  got          : WORD;
  this_version : INTEGER;
  i            : INTEGER;
  o            : TSimObject;
BEGIN
  Assign(F, name);
  Reset(F, 1);
  IF IOResult<>0 THEN BEGIN
    Writeln('ERROR: ', name, ': Cannot open this file.');
    Halt(1);
  END;
  BlockRead(F, h, Sizeof(h), got);
  IF (got<>Sizeof(h)) OR (h.magic[0]<>'C') OR (h.magic[1]<>'C')
   OR (h.magic[2]<>'b') OR (h.magic[3]<>'s') OR (h.magic[4]<>'i')
   OR (h.magic[5]<>'m') OR (h.magic[7]<>#26)
  THEN BEGIN
    Writeln('ERROR: ', name, ': File is invalid.');
    Halt(1);
  END;
  this_version := Ord(h.magic[6]) - Ord('0');
  IF (this_version < 0) OR (this_version > MAX_VERSION) THEN BEGIN
    Writeln('ERROR: ', name, ': Unsupported version ', this_version+1, '.');
    Halt(1);
  END;
  IF this_version > file_version THEN file_version := this_version;

  Writeln('Reading ', name, '...');
  FOR i := 1 TO h.count AND $7FFF DO BEGIN
    FillChar(o, Sizeof(o), 0);
    o.agg := -1;
    o.mass := 100;
    BlockRead(F, o, record_size[this_version], got);
    IF got <> record_size[this_version] THEN BEGIN
      Writeln('ERROR: File is truncated at ship slot ', i, '.');
      Halt(1);
    END;
    AddShip(o);
  END;

  IF h.count AND $8000<>0 THEN BEGIN
    FillChar(o, Sizeof(o), 0);
    o.agg := -1;
    o.mass := 100;
    BlockRead(F, o, record_size[this_version], got);
    IF got <> record_size[this_version] THEN BEGIN
      Writeln('ERROR: File is truncated at planet.');
      Halt(1);
    END;
    AddPlanet(o);
  END;

  IF IOResult<>0 THEN BEGIN
    Writeln('ERROR: I/O error.');
    Halt(1);
  END;

  Close(F);
END;

{ Save everything to a file }
PROCEDURE SaveFile(CONST name : STRING);
VAR
  F            : FILE;
  did          : WORD;
  i            : INTEGER;
CONST h : TNewSimHeader = (magic : 'CCbsim0'#26; count : 0);
BEGIN
  Assign(F, name);
  Rewrite(F, 1);
  IF IOResult<>0 THEN BEGIN
    Writeln('ERROR: ', name, ': Cannot create.');
    Halt(1);
  END;

  Writeln('Writing ', name, '...');

  h.magic[6] := Chr(Ord('0') + file_version);
  h.count := num_ships;
  IF have_planet THEN h.count := h.count OR $8000;
  BlockWrite(F, h, Sizeof(h), did);

  FOR i:=1 TO num_ships DO BEGIN
    BlockWrite(F, ships[i], record_size[file_version], did); 
    IF did<>record_size[file_version] THEN BEGIN
      Writeln('ERROR: Write error - disk full?');
      Halt(1);
    END;
  END;
  IF have_planet THEN BEGIN
    BlockWrite(F, planet, record_size[file_version], did); 
    IF did<>record_size[file_version] THEN BEGIN
      Writeln('ERROR: Write error - disk full?');
      Halt(1);
    END;
  END;
  Close(F);
END;                                                         

VAR
  i, j, n : INTEGER;

BEGIN
  FileMode := 0;
  Writeln('MERGECCB v0.1 - Merge .ccb files - April 2006 by Stefan Reuther');
  n := ParamCount;
  IF n < 2 THEN BEGIN
    Writeln;
    Writeln('Usage: MERGECCB inputfiles.... outputfile');
    Writeln;
    Writeln('This program will take a set of .ccb files (PCC / CCBSim saved');
    Writeln('simulation setups) and merge them to a single file. It supports');
    Writeln('file format versions 1 to 4, i.e. PCC 1.0 up to 1.1.13 (current');
    Writeln('as of April 2006).');
    Writeln;
    Writeln('Send comments, bug reports and questions to <Streu@gmx.de>.');
    Halt;
  END;

  file_version := 0;
  num_ships := 0;

  FOR i := 1 TO n-1 DO LoadFile(ParamStr(i));
  IF (num_ships=0) AND NOT have_planet THEN BEGIN
    Writeln('ERROR: no objects in combined file.');
    Halt(1);
  END;
  SaveFile(ParamStr(n));
  Writeln('Combined file is version ', file_version+1, ' and has ', num_ships, ' ships.');
END.
